@@ -83,10 +83,10 @@ class OrcaNavListGUI:
         selection = self._tree.get_selection()
         selection.select_path(selectedRow)
 
-        btn = dialog.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
+        btn = dialog.add_button(guilabels.BTN_CANCEL, Gtk.ResponseType.CANCEL)
         btn.connect('clicked', self._onCancelClicked)
 
-        btn = dialog.add_button(Gtk.STOCK_JUMP_TO, Gtk.ResponseType.APPLY)
+        btn = dialog.add_button(guilabels.BTN_JUMP_TO, Gtk.ResponseType.APPLY)
         btn.grab_default()
         btn.connect('clicked', self._onJumpToClicked)
 
@@ -102,7 +102,7 @@ class OrcaNavListGUI:
 
     def showGUI(self):
         self._gui.show_all()
-        ts = orca_state.lastInputEventTimestamp
+        ts = orca_state.lastInputEvent.timestamp
         if ts == 0:
             ts = Gtk.get_current_event_time()
         self._gui.present_with_time(ts)
