@@ -771,10 +771,16 @@ class Utilities:
 
         return False
 
+    def isHidden(self, obj):
+        return False
+
     def isLayoutOnly(self, obj):
         """Returns True if the given object is a container which has
         no presentable information (label, name, displayed text, etc.)."""
 
+        if self.isHidden(obj):
+            return True
+
         layoutOnly = False
 
         try:
@@ -1172,11 +1178,13 @@ class Utilities:
                    and ci.getLayer() == pyatspi.LAYER_POPUP:
                     return child
 
-    @staticmethod
-    def pursueForFlatReview(obj):
+    def pursueForFlatReview(self, obj):
         """Determines if we should look any further at the object
         for flat review."""
 
+        if self.isHidden(obj):
+            return False
+
         try:
             state = obj.getState()
         except:
