@@ -170,17 +170,13 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
 
     def generateBraille(self, obj, **args):
         result = []
-        # ARIA widgets get treated like regular default widgets.
-        #
         args['includeContext'] = not self._script.inDocumentContent(obj)
-        args['useDefaultFormatting'] = \
-            self._script.isAriaWidget(obj) \
-            or ((obj.getRole() == pyatspi.ROLE_LIST) \
-                and (not obj.getState().contains(pyatspi.STATE_FOCUSABLE)))
 
         oldRole = None
         if self._script.utilities.isEntry(obj):
             oldRole = self._overrideRole(pyatspi.ROLE_ENTRY, args)
+        elif self._script.utilities.isClickableElement(obj):
+            oldRole = self._overrideRole(pyatspi.ROLE_LINK, args)
 
         # Treat menu items in collapsed combo boxes as if the combo box
         # had focus. This will make things more consistent with how we
@@ -195,7 +191,6 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
         result.extend(braille_generator.BrailleGenerator.\
                           generateBraille(self, obj, **args))
         del args['includeContext']
-        del args['useDefaultFormatting']
         if oldRole:
             self._restoreRole(oldRole, args)
         return result
