@@ -103,6 +103,11 @@ class Script(default.Script):
             orca.setLocusOfFocus(event, event.source)
             return
 
+        # The above issue also seems to happen with spin buttons.
+        if role == pyatspi.ROLE_SPIN_BUTTON:
+            orca.setLocusOfFocus(event, event.source)
+            return
+
         # https://bugzilla.gnome.org/show_bug.cgi?id=720987
         if role == pyatspi.ROLE_TABLE_COLUMN_HEADER:
             orca.setLocusOfFocus(event, event.source)
