@@ -27,7 +27,6 @@ __copyright__ = "Copyright (c) 2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 from .orca_i18n import C_
-from . import orca_state
 
 # Translators: this is a structure to assist in the generation of
 # localized strings for the various text attributes. 
@@ -755,16 +754,13 @@ def getTextAttributeKey(localizedTextAttr):
     localized text attribute.
     """
 
-    if isinstance(localizedTextAttr, unicode):
-        localizedTextAttr = localizedTextAttr.encode("UTF-8")
-
     for key, value in list(_textAttributeTable.items()):
         if value == localizedTextAttr:
             return key
 
     return localizedTextAttr
 
-def getTextAttributeName(textAttr):
+def getTextAttributeName(textAttr, script=None):
     """Given a text attribute, returns its localized equivalent.
 
     Arguments:
@@ -774,16 +770,9 @@ def getTextAttributeName(textAttr):
     attribute.
     """
 
-    if isinstance(textAttr, unicode):
-        textAttr = textAttr.encode("UTF-8")
-
     # Normalize the name to an Atk name before attempting to look it up.
     #
-    if orca_state.activeScript:
-        textAttr = orca_state.activeScript.getAtkNameForAttribute(textAttr)
-
-    try:
-        return _textAttributeTable[textAttr]
-    except:
-        return textAttr
+    if script:
+        textAttr = script.getAtkNameForAttribute(textAttr)
 
+    return _textAttributeTable.get(textAttr, textAttr)
